<?php

/**
 * @file
 * Alters callbacks for nodequeue_arrange_subqueue_form form.
 */

/**
 * Submit function for nodequeue_scheduler_alter_subqueue_form
 * on 'Reverse' button.
 */
function nodequeue_scheduler_alter_subqueue_form_reverse_submit($form, &$form_state) {
  nodequeue_scheduler_alter_subqueue_form_submit($form, $form_state, TRUE, FALSE);
}

/**
 * Submit function for nodequeue_scheduler_alter_subqueue_form
 * on 'Shuffle' button.
 */
function nodequeue_scheduler_alter_subqueue_form_shuffle_submit($form, &$form_state) {
  nodequeue_scheduler_alter_subqueue_form_submit($form, $form_state, FALSE, TRUE);
}

/**
 * Submit handler for nodequeue drag'n'drop form.
 *
 * Updates node positions in {nodequeue_nodes}.
 */
function nodequeue_scheduler_alter_subqueue_form_submit($form, &$form_state, $reverse = FALSE, $shuffle = FALSE) {
  $nodes = array();
  foreach ($form_state['values'] as $nid => $element) {
    if (is_numeric($nid)) {
      $nodes[$form_state['values'][$nid]['position']] = array(
        'nid' => $nid,
        'data' => $form_state['values'][$nid],
      );
    }
  }

  $message = t('The queue has been updated.');

  if ($reverse || $shuffle) {
    $keys = array_keys($nodes);
    $values = array_values($nodes);

    // Reverse the list if the reverse button was pressed.
    if ($reverse) {
      $values = array_reverse($values);
      $message = t('The queue has been reversed.');
    }

    // Shuffle the list if the shuffle button was pressed.
    if ($shuffle) {
      shuffle($values);
      $message = t('The queue has been shuffled.');
    }

    $nodes = array_combine($keys, $values);
  }

  $qid  = $form['#queue']['qid'];
  $sqid = $form['#subqueue']['sqid'];
  nodequeue_scheduler_alter_save_subqueue_order($nodes, $qid, $sqid);

  drupal_set_message(check_plain($message));
}

/**
 * Validates new subqueue order information and if it passes validation,
 * deletes the old subqueue data from the database and saves the new data.
 *
 * @param array $nodes
 *   An array of nodes, keyed on the subqueue position.
 * @param integer $qid
 *   The queue id.
 * @param integer $sqid
 *   The subqueue id.
 *
 * @return array
 *   Where the first element is a numeric status code (0 means successfully
 *   saved) and the second element is a status message.
 */
function nodequeue_scheduler_alter_save_subqueue_order($nodes, $qid, $sqid) {
  $positions = array();
  $now = time();

  $queue = nodequeue_load($qid);
  $subqueue = nodequeue_load_subqueue($sqid);

  // Cleanup the node array.
  $clean = array();
  $count = 1;
  ksort($nodes);
  module_invoke_all('nodequeue_sort_alter', $sqid, $nodes);
  foreach ($nodes as $pos => $node) {
    if (!is_numeric($node['nid']) || $node['nid'] < 1) {
      return array(NODEQUEUE_INVALID_NID, 'Invalid nid value. New subqueue order not saved.');
    }
    if (is_numeric($pos)) {
      $clean[$count] = $node;
      $count++;
    }
    elseif ($pos == 'r') {
      // Call nodequeue_remove hook.
      module_invoke_all('nodequeue_remove', $sqid, $node['nid']);
    }
    else {
      return array(NODEQUEUE_INVALID_POSITION, 'Invalid position value. New subqueue order not saved.');
    }
  }
  $nodes = $clean;

  array_walk($clean, create_function('&$node', '$node = serialize($node);'));

  if (count(array_unique($clean)) < count($nodes)) {
    return array(NODEQUEUE_DUPLICATE_POSITION, 'Duplicate position values are not allowed. New subqueue order not saved.');
  }
  // Allow other modules to alter the order of nodes.
  foreach (module_implements('nodequeue_save_subqueue_order_alter') as $module) {
    $function = $module . '_nodequeue_save_subqueue_order_alter';
    $function($sqid, $nodes);
  }

  // DON"T CLEAR the queue.
  // Save the new positions.
  foreach ($nodes as $pos => $node) {
    $args = array();
    if ($pos != 'r') {
      $positions[] = $pos;
      $placeholders = ' (%d, %d, %d, %d)';
      $args = array($pos, $sqid, $qid, $node['nid']);
    }

    if (!empty($args)) {
      // DON'T INSERT JUST UPDATE INSTEAD.
      $sql = 'UPDATE {nodequeue_nodes} SET position = %d WHERE sqid = %d AND qid = %d AND nid = %d';
      db_query($sql, $args);
    }
  }

  if ($queue->size) {
    // Only necessary if the subqueue is of finite length.
    nodequeue_check_subqueue_size($queue, $subqueue);
  }

  return array(NODEQUEUE_OK, 'The queue has been updated.');
}
